/*************************************************************
 * Pocket oscilloscope; (c) 2009 Hynek Sladky                *
 *                                                           *
 * adc.c                                                     *
 *************************************************************/
#include "global.h"
#include "adc.h"

// ====================================
// the simplest ADC function
// ------------------------------------
unsigned int getADraw (unsigned int channel) {
  // convert single input
  ADC12CTL1 &= 0x0FF9;
  ADC12CTL0 &= 0x00FF;
  //ADC12CTL0 |= 0x6600; // 128 cycles = 32us - required by temperature sensor
  //ADC12CTL0 |= 0xFF00; // 1024 cycles = 256us!!!
  //ADC12CTL0 = SHT1_15 | SHT0_15 | REF2_5V | REFON | ADC12ON; // sample 8 cycles, REF 2.5V, multiple sample
  ADC12CTL0 = SHT1_15 | SHT0_15 | REFON | ADC12ON; // sample 8 cycles, REF 2.5V, multiple sample
  ADC12CTL1 = SHS_0 | SHP | ADC12SSEL_2 | CONSEQ_0; // start at address 0, start form ADC12SC, sample for SHT time, MCLK
  // set sequence requests
  ADC12MCTL0 = channel | SREF_1 | EOS;
  // start conversion
  ADC12CTL0 |= ADC12SC | ENC;
  ADC12CTL0 &= ~ADC12SC;
  // wait for completion
  while (ADC12CTL1 & ADC12BUSY);
  // switch ADC conversion off
  ADC12CTL0 &= ~ENC;
  return ADC12MEM0;
}
// ------------------------------------
// buffer for data
// ------------------------------------
extern volatile int ADC_buf[128];
volatile unsigned int ADC_pos;
volatile unsigned int ADC_trigstat;
extern volatile unsigned int ADC_int_stat;
// local variables - but accessible in ASM functions
volatile int ADC_cnt, ADC_pre;
int ADC_trigval;
// ------------------------------------
// start ADC measurement
// ------------------------------------
// C code works up to maximum sampling rate: MCLK/1, SHT=4: 81.818 kS/s
// ASM code works up to maximum sampling rate: MCLK/1, SHT=3: 140 kS/s
//   with SHT=1 cca 16px drawn, with SHT=2 cca 64px is drawn
//   but important to mention, that cca 3px are influenced by the signal if change from L to H
//   next finding: with shortening sampling time, it is mode finger touch resistant; needs stronger signal
// after last modifications doesn't work interrupt even with 50kSps - but maybe there is another reason...
#define MAX_SAMPLE_RATE   ADC_20kSps
// ------------------------------------
void ADC_start (unsigned char channel, unsigned int post, unsigned int trig, unsigned int rate, unsigned int flags) {
int j;
  // ADC init
  // ADC on, ref int
  ADC12CTL0 &= ~(ENC | ADC12SC);
  ADC12IE = 0;
  //ADC12CTL0 = SHT1_1 | SHT0_1 | REF2_5V | REFON | ADC12ON; // sample 8 cycles, REF 2.5V, multiple sample
  ADC12CTL0 = SHT1_1 | SHT0_1 | REFON | ADC12ON; // sample 8 cycles, REF 2.5V, multiple sample
  ADC12CTL1 = CSTARTADD_0 | SHS_0 | SHP | ADC12SSEL_2 | CONSEQ_2; // start at address 0, start form ADC12SC, sample for SHT time, MCLK
  // timer A setup
  TACCR0 = rate;
  TACCR1 = rate>>1;
  TACCTL0 = 0;
  TACCTL1 = OUTMOD_3;
  TACTL = TASSEL_2 | ID_0 | MC_1; // SMCLK/1 up to TACCR0
  TAR = 0;
  if (rate <= ADC_100us) {
    // repeat single channel sampling - maximum rate
    ADC12CTL0 |= MSC; // fastest conversion w/o delay
    ADC12MCTL0 = SREF_1 | channel | EOS;
  } else
  if (rate < MAX_SAMPLE_RATE) {
    // repeat single channel sampling
    ADC12CTL1 |= SHS_1; // start from TA.OUT1
    ADC12MCTL0 = SREF_1 | channel | EOS;
  } else {
    // repeat sequence of channels
    ADC12CTL1 |= SHS_1 | CONSEQ_3;
    ADC12MCTL0 = SREF_1 | channel;
    ADC12MCTL1 = SREF_1 | channel;
    ADC12MCTL2 = SREF_1 | channel;
    ADC12MCTL3 = SREF_1 | channel;
    ADC12MCTL4 = SREF_1 | channel;
    ADC12MCTL5 = SREF_1 | channel;
    ADC12MCTL6 = SREF_1 | channel;
    ADC12MCTL7 = SREF_1 | channel; // | EOS;
    ADC12MCTL8 = SREF_1 | channel;
    ADC12MCTL9 = SREF_1 | channel;
    ADC12MCTL10 = SREF_1 | channel;
    ADC12MCTL11 = SREF_1 | channel;
    ADC12MCTL12 = SREF_1 | channel;
    ADC12MCTL13 = SREF_1 | channel;
    ADC12MCTL14 = SREF_1 | channel;
    ADC12MCTL15 = SREF_1 | channel; // | EOS;
  }
  // variables setup
  ADC_pos = 0;
  ADC_cnt = post;
  ADC_pre = (post<128) ? 128-post : 0;
  ADC_trigval = trig;
  ADC_trigstat = 0;
  if (flags & ADC_ROLL_MODE) {
    ADC_int_stat = 36;
    ADC_pre = ADC_cnt;
    for (j=0; j<128; j++) {
      ADC_buf[j] = trig;
    }
  } else
  if (flags & ADC_FREE_MODE)
    ADC_int_stat = 32;
  else {
    ADC_int_stat = 0;
    if (flags & ADC_AUTO_MODE) ADC_int_stat += 16;
    if (flags & ADC_FALL_EDGE) ADC_int_stat += 8;
  }
  // regarding requested samplerate, HW is set up and run...
  if (rate < MAX_SAMPLE_RATE) {
    // encoder detection setup
    P2IES = (P2IES & ~(KEY_MASK | INC_MASK)) | (KEY_PORT & (KEY_MASK | INC_MASK));
    P2IFG = 0;
    // start sampling
    ADC12CTL0 |= ENC | ADC12SC;
    // use fast dedicated function
    ADC_sample ();
  } else
  {
    // set interrupt
    __disable_interrupt ();
    ADC12IFG = 0x0000;
    ADC12IE = 0x8080;
    ADC12CTL0 |= ADC12OVIE; // | ADC12TOVIE; // overflow interrupt enable
    while (ADC12IV) {
      // clear pending interrupt request
      ADC12IFG = 0;
      ADC12MEM14 = ADC12MEM15;
      ADC12MEM6 = ADC12MEM7;
    }
    // start sampling
    ADC12CTL0 |= ENC | ADC12SC;
    //ADC12CTL0 &= ~ADC12SC;
    __enable_interrupt ();
  }
}
// ------------------------------------
// test if sampling is active
// ------------------------------------
int ADC_state (void) {
  // check if error
  if (ADC_trigstat & 0xFFF0) return ADC_trigstat;
  // ROLL mode running
  //if (ADC_int_stat==36) return ADC_ROLL_RUN;
  // not finished yet
  if (ADC_cnt) return 0;
  // return completed flag - cant be value of zero!
  return ADC_trigstat | ADC_FINISHED;
}
// ------------------------------------
// stop sampling
// ------------------------------------
void ADC_stop (void) {
  __disable_interrupt ();
  // set EOS bit
  ADC12MCTL0 |= EOS;
  // clear ENC bit & disable interrupts
  ADC12CTL0 &= ~0x000F;
  // set single channel single conversion and start from ADC12SC bit
  ADC12CTL1 &= ~(CONSEQ_3 | SHS_3);
  // wait for completion
  while (ADC12CTL1 & ADC12BUSY);
  // disable & clear interrupt requests
  ADC12IE = 0;
  ADC12IFG = 0;

  while (ADC12IV) {
    // clear pending interrupt request
    ADC12IFG = 0;
    ADC12MEM14 = ADC12MEM15;
    ADC12MEM6 = ADC12MEM7;
  }
  // disable multiple sample
  ADC12CTL0 &= ~MSC;
  // dummy conversion
  ADC12CTL0 |= ADC12SC | ENC;
  //ADC12CTL0 &= ~ADC12SC;
  while (ADC12CTL1 & ADC12BUSY);
  ADC12CTL0 &= ~ENC;
  // result must be read!!!
  ADC12MEM1 = ADC12MEM0;
  __enable_interrupt ();
  ADC_int_stat=0;
}
// ------------------------------------
// ADC interrupt
// ------------------------------------
// works up to maximum sampling frequency: MCLK/1, SHT=4: 81.818 kS/s
#ifdef _ADC_INT_C_
void ADC_interrupt (void) __interrupt[ADC12_VECTOR] {
unsigned int *ptr;
int i, val;
  switch (ADC12IV) {
    default: // unexpected interrupt
      _trigstat = 0x10;
      return;
    case 0x02: // ADC12MEMx overflow
      _trigstat = 0x12;
      return;
    case 0x04: // conversion time overflow
      _trigstat = 0x14;
      return;
    case 0x14: // ADC12MEM7
      ptr = (unsigned int*)&ADC12MEM0;
      break;
    case 0x24: // ADC12MEM15
      ptr = (unsigned int*)&ADC12MEM8;
      break;
  }
  for (i=8; i; i--) {
    val = *ptr++;
    if (_cnt) {
      if (ADC_pos>=128) ADC_pos=0;
      _adcbuf[ADC_pos++] = val;
      if (!_trigstat) {
        if (_pre) _pre--;
        else
        if (val<_trigval-TRIG_HYST) _trigstat = 1;
      } else
      if (_trigstat&1) {
        if (val>_trigval) _trigstat = 2;
      } else
        _cnt--;
    }
  }
}
#endif
