/*************************************************************
 * Pocket oscilloscope; (c) 2009 Hynek Sladky                *
 *                                                           *
 * adc.h                                                     *
 *************************************************************/
#ifndef _ADC_H_
#define _ADC_H_
// ====================================
// constants
// ------------------------------------
/**/
// sample rate
#define ADC_300kSps   0     // 21
#define ADC_100kSps   63
#define ADC_50kSps    126
#define ADC_20kSps    315
#define ADC_10kSps    630
#define ADC_5kSps     1260
#define ADC_2kSps     3150
#define ADC_1kSps     6300
#define ADC_500Sps    12600
#define ADC_200Sps    31500
#define ADC_100Sps    63000
/**/
// time/divison (division = 30px)
#define ADC_100us     21
#define ADC_200us     42
#define ADC_500us     105
#define ADC_1ms       210
#define ADC_2ms       420
#define ADC_5ms       1050
#define ADC_10ms      2100
#define ADC_20ms      4200
#define ADC_50ms      10500
#define ADC_100ms     21000
#define ADC_200ms     42000
#define ADC_250ms     52500   // for ROLL mode
// ADC offset (for ASM function)
//#define ADC_OFFSET    2048    // theoretically default state
#define ADC_OFFSET    2031    // real value after HW modification
//#define ADC_OFFSET    1011    // without signal, should be cca 1024
//#define ADC_OFFSET    2019    // input = GND
// ====================================
// data structures
// ------------------------------------
// ====================================
// global variables
// ------------------------------------
#ifndef _MSP430_ASM_
extern volatile int ADC_buf[128];
//extern unsigned char TMP_buf[256]; // points to same variable as ADC_buf!!!
extern unsigned char TMP_buf[256]; // points to second half of FFT_buf...
extern int FFT_buf[128][2]; // points to the same buffer as ADC_buf!!!
extern volatile unsigned int ADC_pos;
//extern volatile unsigned int ADC_trigstat;
//extern volatile unsigned int ADC_int_stat;
#endif
// ADC flags
#define ADC_RISE_EDGE   0x0000
#define ADC_FALL_EDGE   0x0001
#define ADC_EDGE_MASK   0x0001
#define ADC_NORMAL_MODE 0x0000
#define ADC_AUTO_MODE   0x0010
#define ADC_FREE_MODE   0x0020
#define ADC_ROLL_MODE   0x0040
#define ADC_MODE_MASK   0x0070
// ADC status (ADC_trigstat)
#define ADC_FINISHED    0x0001
#define ADC_TRIGGERED   0x0002
//#define ADC_ROLL_RUN    0x0004
#define ADC_BUF_PART    0x0080
#define ADC_ERR_OVER    0x0100
#define ADC_ERR_TIME    0x0200
#define ADC_ERR_INT     0x0400
// ====================================
// function prototypes
// ------------------------------------
#ifndef _MSP430_ASM_
// assembler functions
int ADC_sample (void);
unsigned int ADC_convert (unsigned int sample, unsigned int multiply);
// C functions
unsigned int getADraw (unsigned int channel);
void ADC_start (unsigned char channel, unsigned int post, unsigned int trig, unsigned int rate, unsigned int flags);
int ADC_state (void);
void ADC_stop (void);
void ADC_get (int *buf);
#endif
#endif
