/*************************************************************
 * Pocket oscilloscope; (c) 2009 Hynek Sladky                *
 *                                                           *
 * fft.c                                                     *
 *************************************************************/
#include <stdio.h>
#include "global.h"
#include "fft.h"
// ====================================
// FFT
// ====================================
// ------------------------------------
// compile control
// ------------------------------------
#define FFT_PREORDER
// ------------------------------------
// macros
// ------------------------------------
#define	FFT_N		128
//#define	FFT_SHIFT	12//10
#define	FP(a)		(int)(a*(1<<FFT_SHIFT))
#define SWAP(a,b)	tempr=(a);(a)=(b);(b)=tempr
// ------------------------------------
// sample reordering table
// ------------------------------------
#ifdef FFT_PREORDER
const unsigned char sample_order[FFT_N] = {
  0, 64, 32, 96, 16, 80, 48, 112,
  8, 72, 40, 104, 24, 88, 56, 120,
  4, 68, 36, 100, 20, 84, 52, 116,
  12, 76, 44, 108, 28, 92, 60, 124,
  2, 66, 34, 98, 18, 82, 50, 114,
  10, 74, 42, 106, 26, 90, 58, 122,
  6, 70, 38, 102, 22, 86, 54, 118,
  14, 78, 46, 110, 30, 94, 62, 126,
  1, 65, 33, 97, 17, 81, 49, 113,
  9, 73, 41, 105, 25, 89, 57, 121,
  5, 69, 37, 101, 21, 85, 53, 117,
  13, 77, 45, 109, 29, 93, 61, 125,
  3, 67, 35, 99, 19, 83, 51, 115,
  11, 75, 43, 107, 27, 91, 59, 123,
  7, 71, 39, 103, 23, 87, 55, 119,
  15, 79, 47, 111, 31, 95, 63, 127
};
#endif
// ====================================
// square root
// ------------------------------------
// magnitude estimator - based on comp.dsp work
int root_i (int a, int b) {
int c;
  // the lowest peak error:
  // 0.960433870103*max + 0.397824734759*min
  // low peak error:
  // 31/32 max + 3/8 min
  if (a<0) a=-a;
  if (b<0) b=-b;
  if (a<b) {c=a; a=b; b=c;}
  a-=a>>5;
  b>>=1;
  return a+b-(b>>2);
}
// ------------------------------------
// integer FFT
// ------------------------------------
void fft_Bi (int *data, int mode) {
int j, i, ip, le, le2, k;
int ur, ui, tr, ti;
int shift;
const int s[7][2] = {
  {FP (-1.000000), FP (0.000000)},
  {FP (0.000000), FP (-1.000000)},
  {FP (0.707107), FP (-0.707107)},
  {FP (0.923880), FP (-0.382683)},
  {FP (0.980785), FP (-0.195090)},
  {FP (0.995185), FP (-0.098017)},
  {FP (0.998795), FP (-0.049068)}
};

  // data normalization
  k=0x7FFF;
  for (i=j=0; i<FFT_N; i++) {
    if (data[i]<k) k=data[i];
    if (data[i]>j) j=data[i];
  }
  // amplitude
  j-=k;
  shift=0;
  while (j>127) {
    j>>=1;
    shift++;
  }
  // DC offset
  k+=shift>>1;
  // data loading and ordering
  for (i=0; i<FFT_N; i++) {
    FFT_buf[sample_order[i]][0]=(data[i]-k)>>shift;
    FFT_buf[sample_order[i]][1]=0;
  }
  // loop for each stage
  k=0;
  for (le=2; le<=FFT_N; le<<=1) {
    le2 = le>>1;
    ur = 1<<FFT_SHIFT;
    ui = 0;
    // loop for each sub DFT
    for (j=0; j<le2; j++) {
      // loop for each butterfly
      for (i=j; i<FFT_N; i+=le) {
	ip = i+le2;
	// butterfly calculation
	//tr = ((FFT_buf[ip][0]*ur)>>FFT_SHIFT) - ((FFT_buf[ip][1]*ui)>>FFT_SHIFT);
	//ti = ((FFT_buf[ip][0]*ui)>>FFT_SHIFT) + ((FFT_buf[ip][1]*ur)>>FFT_SHIFT);
	tr = FFT_mul (FFT_buf[ip][0], ur) - FFT_mul (FFT_buf[ip][1], ui);
	ti = FFT_mul (FFT_buf[ip][0], ui) + FFT_mul (FFT_buf[ip][1], ur);
	FFT_buf[ip][0] = FFT_buf[i][0] - tr;
	FFT_buf[ip][1] = FFT_buf[i][1] - ti;
	FFT_buf[i][0] += tr;
	FFT_buf[i][1] += ti;
      }
      tr = ur;
      ur = FFT_mul (tr, s[k][0]) - FFT_mul (ui, s[k][1]);
      ui = FFT_mul (tr, s[k][1]) + FFT_mul (ui, s[k][0]);
    }
    k++;
  }
  // finalize unnorming /FFT_N - include correction for vcetne zapocitani korekce komplexne-sdruzenych FFT vysledku (neresi DC a Fs/2)
  shift = 6-shift;
  // clear DC - not necessary to display it
  FFT_buf[0][0] = FFT_buf[0][1] = 0;
  // Fs/2 can be left unchanged because it stays unvisible
  // in fact DC and fs/2 cover narrower interval so have twice big amplitude...
  // computing amplitude & storing results
  // meanwhile easy abs sum
  if (mode==1) {
    // half signal, half FFT
    //for (i=0; i<FFT_N/2; i++) {
    for (i=0; i<61; i++) {
      //data[FFT_N/2+i]=((FFT_buf[i][0]<0)?-FFT_buf[i][0]:FFT_buf[i][0])+((FFT_buf[i][1]<0)?-FFT_buf[i][1]:FFT_buf[i][1]);
      //data[FFT_N/2+i]=root_i (FFT_buf[i][0], FFT_buf[i][1]);
      data[67+i]=root_i (FFT_buf[i][0], FFT_buf[i][1]) >> shift;
    }
  } else {
    // FFT on whole screen
    //for (i=0; i<FFT_N/2; i++) {
    for (i=0; i<61; i++) {
      //data[i+i]=((FFT_buf[i][0]<0)?-FFT_buf[i][0]:FFT_buf[i][0])+((FFT_buf[i][1]<0)?-FFT_buf[i][1]:FFT_buf[i][1]);
      data[i+i+7]=root_i (FFT_buf[i][0], FFT_buf[i][1]) >> shift;
    }
    // interpolate missing values
    //for (i=1; i<FFT_N; i+=2) {
    for (i=8; i<128; i+=2) {
      //data[i]=(data[i-1]+data[i+1])>>1; // prekladac 1.3 to neprelozi korektne!!!
      data[i]=(data[i+1]+data[i-1])>>1;
    }
  }
}
