/*************************************************************
 * Pocket oscilloscope; (c) 2009 Hynek Sladky                *
 *                                                           *
 * hardware.h                                                *
 *************************************************************/
#ifndef _HARDWARE_H_
#define _HARDWARE_H_
// ====================================
// hardware version selection
// ------------------------------------
#define LH10001
// ====================================
// hardware definition for version 01
// ------------------------------------
#ifdef LH10001
// power management
#define VCC_EN_P    P6OUT
#define VCC_EN_B    BIT6
#define LED_EN_P    P6OUT
#define LED_EN_B    BIT7
#define WDI_P       P1OUT
#define WDI_B       BIT1
#define ADC_VCC     INCH_0
#define ADC_INP     INCH_1
#define ADC_RFID    INCH_2
// LED latch
#define LED_DTA_P   P6OUT
#define LED_DTA_B   BIT4
#define LED_CLK_P   P6OUT
#define LED_CLK_B   BIT5
#define LED_LATCH_P P4OUT
#define LED_LATCH_B BIT0
#define LED_OE_P    P4OUT
#define LED_OE_B    BIT5
// SPI flash
#define MM_WP_P     P5OUT
#define MM_WP_B     BIT0
#define MM_SI_P     P5OUT
#define MM_SI_B     BIT1
#define MM_SO_P     P5IN
#define MM_SO_B     BIT2
#define MM_SCK_P    P5OUT
#define MM_SCK_B    BIT3
#define MM_CS_P     P5OUT
#define MM_CS_B     BIT4
// LCD (controller ST7565R)
#define LCD_DTA_P   MM_SI_P
#define LCD_DTA_B   MM_SI_B
#define LCD_CLK_P   MM_SCK_P
#define LCD_CLK_B   MM_SCK_B
#define LCD_CS_P    P5OUT
#define LCD_CS_B    BIT5
#define LCD_RST_P   P5OUT
#define LCD_RST_B   BIT6
#define LCD_A0_P    P5OUT
#define LCD_A0_B    BIT7
// port reset values
#define POR_P1OUT 0b10000000
#define POR_P1DIR 0b11110101 // WDI necham plovouci
#define POR_P2OUT 0b00000000
#define POR_P2DIR 0b10001111 //0b11111111
#define POR_P3OUT 0b00000000
#define POR_P3DIR 0b11111111
#define POR_P4OUT 0b00000000
#define POR_P4DIR 0b11111111
#define POR_P5OUT 0b01010000
#define POR_P5DIR 0b11111011
#define POR_P6OUT 0b01000000
#define POR_P6DIR 0b11111000
#define POR_P6SEL 0b00000111
// incremental decoder
#define CTL_KEY_P   P2IN
#define CTL_KEY_B   BIT4
#define CTL_INC_P   P2IN
#define CTL_INC_B   BIT5
#define CTL_DEC_P   P2IN
#define CTL_DEC_B   BIT6
#define KEY_PORT    P2IN
#define KEY_MASK    0x0010
#define INC_PORT    P2IN
#define INC_MASK    0x0060
#define INC_SHIFT   5
// 1/10 switch
#define DIV_10_P    P1OUT
#define DIV_10_B    BIT7
// PWM
#define PWM_R_CTL   TBCCTL1
#define PWM_R_CR    TBCCR1
#define PWM_R_SEL   0x02
#define PWM_G_CTL   TBCCTL3
#define PWM_G_CR    TBCCR3
#define PWM_G_SEL   0x08
#define PWM_B_CTL   TBCCTL2
#define PWM_B_CR    TBCCR2
#define PWM_B_SEL   0x04
#define PWM_PWR_CTL TBCCTL4
#define PWM_PWR_CR  TBCCR4
#define PWM_PWR_SEL 0x10
#endif

#endif
