/*************************************************************
 * Pocket oscilloscope; (c) 2009 Hynek Sladky                *
 *                                                           *
 * lcd.c                                                     *
 *************************************************************/
#include "global.h"
#include <string.h>
#include "lcd.h"
// fonts definition
// proportional font
#include "Font_8x6_p.inc"
// fixed-width font
//#include "Font_8x6.inc"
// ====================================
// compile control
// ------------------------------------
//#define SW_SPI // SPI is software driven
#define HW_SPI // SPI is hardware driven
//#define HW_WAIT // wait after every byte for incoming data
// ------------------------------------
// variable for current LCD state
// ------------------------------------
static LCDstateType LCDstate = {0, 0, 0, 0, 0, 0, {0, {0, 0, 0, 0}}};
#define LCD_ROWS  8
#define LCD_COLS  132
// ====================================
// generic delay
// ------------------------------------
void LCDdelay (unsigned ms) {
  TACTL = TASSEL1 | ID1 | MC1; // SMCLK/4 up to FFFFh
  TACCR2 = TAR;
  while (ms) {
    TACCR2 += 1575;
    TACCTL2 &= ~CCIFG;
    ms--;
    while (!(TACCTL2 & CCIFG));
  }
}
// ====================================
// LCD utilities
// model: 64128L
// driver: Sitronix ST7565R
// ====================================
// write byte to LCD through SPI
// ------------------------------------
#ifdef SW_SPI
void LCDbyte (unsigned char data) {
int i;
  for (i=8; i; i--) {
    if (data & 0x80) LCD_DTA = 1;
    else LCD_DTA = 0;
    data <<= 1;
    LCD_CLK = 0;
    LCD_CLK = 1;
  }
}
#endif
#ifdef HW_SPI
void LCDbyte(unsigned char data) {
unsigned char c;
  c = U1RXBUF;
  while (!(IFG2 & UTXIFG1));
  U1TXBUF = data;
  // if not waiting for receive - 4.5us per byte!!!
#ifdef HW_WAIT
  // if waiting for receive - 11.1us per byte (transmission takes 2.4us)
  while (!(U1TCTL & TXEPT));
  while (!(IFG2 & URXIFG1));
  data = U1RXBUF; // works but is slow for long blocks of data
#endif
}
// ====================================
// SPI init
// ------------------------------------
void SPI_init (void) {
  U1CTL |= SWRST;
  U1CTL |= CHAR | SYNC | MM;
  U1TCTL |= CKPL | SSEL1 | STC;
  U1BR0 = 0x02;
  U1BR1 = 0x00;
  U1MCTL = 0x00;
  ME2 |= USPIE1;
  P5SEL |= 0x0E;
  U1CTL &= ~SWRST;
}
#endif
// ====================================
// SPI end
// ------------------------------------
void LCDend (void) {
#ifdef HW_SPI
#ifndef HW_WAIT
  while (!(IFG2 & UTXIFG1));
  while (!(U1TCTL & TXEPT));
  while (!(IFG2 & URXIFG1));
#endif
#endif
  LCD_CS_P |= LCD_CS_B;
}
// ====================================
// write 1-byte command
// ------------------------------------
void LCDcmd (unsigned char cmd) {
  LCD_CS_P &= ~LCD_CS_B;
  LCD_A0_P &= ~LCD_A0_B;
  LCDbyte (cmd);
  LCDend ();
}
// ====================================
// write 2-byte command
// ------------------------------------
void LCDcmd2 (unsigned char cmd, unsigned char data) {
  LCD_CS_P &= ~LCD_CS_B;
  LCD_A0_P &= ~LCD_A0_B;
  LCDbyte (cmd);
  LCDbyte (data);
  LCDend ();
}
// ====================================
// start writing data
// ------------------------------------
void LCDdata (unsigned char line, unsigned char col) {
  LCD_CS_P &= ~LCD_CS_B;
  LCD_A0_P &= ~LCD_A0_B;

  //ro_ = (LCDstate.row;

  //LCDbyte (0x40); // clear vertical scroll position
  //LCDbyte (0xB0 + (line & 0x0F)); // select line
  LCDbyte (0xB0 + ((line + LCDstate.ofs) & (LCD_ROWS-1))); // select line
  LCDbyte (0x10 + (col >> 4)); // set column position
  LCDbyte (0x00 + (col & 0x0F));
#ifdef HW_SPI
#ifndef HW_WAIT
  while (!(IFG2 & UTXIFG1));
  while (!(U1TCTL & TXEPT));
  while (!(IFG2 & URXIFG1));
#endif
#endif
  LCD_A0_P |= LCD_A0_B;
}
// ====================================
// start writing data
// ------------------------------------
void LCD_init_HW (void) {

#ifdef HW_SPI
  // SPI HW init
  SPI_init ();
#endif
  // timer init
  TACTL = TASSEL1 | ID1 | MC1;
  TACCTL2 = 0;
  // switch power for LCD
  VCC_EN_P &= ~VCC_EN_B;
  // reset LCD
  LCD_RST_P &= ~LCD_RST_B;
  LCDdelay (100);
  // start initialization
  LCD_CS_P |= LCD_CS_B;
  LCD_DTA_P &= ~LCD_DTA_B;
  LCD_CLK_P |= LCD_CLK_B;
  LCD_A0_P &= ~LCD_A0_B;
  LCDdelay (1);
  LCD_RST_P |= LCD_RST_B;
  LCDdelay (1);
  // send parameters
  LCDcmd (0x40); // start line 0
  LCDcmd (0xA0); // ADC reverse
  LCDcmd (0xC8); // normal COM0-COM63
  LCDcmd (0xA6); // display normal
  LCDcmd (0xA2); // bias 1/9 (duty 1/65)
  LCDcmd (0x2F); // booster on, regulator on, follower on
  LCDcmd2 (0xF8, 0x00); // booster 4x
  LCDcmd (0x20+6); // V0 voltage
  //LCDcmd2 (0x81, 0x20); // contrast
  LCDcmd2 (0x81, 0x10); // contrast
  //LCDcmd (0x40); // start line
  // clear LCD

  // enable display
  LCDcmd (0xAF); // LCD ON
}
// ------------------------------------
// erase LCD row
// ------------------------------------
void LCDclearRow (unsigned char row) {
int i;
  LCDdata (row, 0);
  for (i=LCD_COLS; i; i--)
    LCDbyte (0);
  LCDend ();
}
// ------------------------------------
// set current position
// ------------------------------------
void LCDpos (unsigned char co_, unsigned char ro_) {

  LCDstate.col=co_;
  LCDstate.row=ro_;
  if (ro_>=LCD_ROWS) {
    LCDstate.ofs++;
    LCDstate.row=LCD_ROWS-1;
    if (LCDstate.ofs>=LCD_ROWS)
      LCDstate.ofs=0;
    LCDcmd (0x40 | (LCDstate.ofs << 3));
    LCDclearRow (LCDstate.row + LCDstate.ofs);
  }
  // immediately start communication
  //ro_ = (LCDstate.row+LCDstate.ofs) & (LCD_ROWS-1);
  //LCDdata (ro_, co_);
}
// ------------------------------------
// erase whole LCD
// ------------------------------------
void LCDclear (void) {
unsigned char lrow;
  for (lrow=0; lrow<LCD_ROWS; lrow++) {
    LCDclearRow (lrow);
  }
  LCDpos (0, 0);
  LCDstate.ofs=0;
}
// ------------------------------------
// write data from buffer to LCD
// ------------------------------------
void LCDbufw (const unsigned char *buf, unsigned char len, unsigned char step) {

  LCDpos (LCDstate.col, LCDstate.row);
  LCDdata (LCDstate.row, LCDstate.col);
  while (len) {
    LCDbyte ((*buf ^ LCDstate.xor) | LCDstate.or);
    LCDstate.col++;
    buf+=step;
    len--;
  }
  LCDend ();
}
// ------------------------------------
// repeat requested byte write to LCD (erasing)
// ------------------------------------
void LCDfill (char data, unsigned char to_x) {

  if (LCDstate.col > to_x) return;
  LCDpos (LCDstate.col, LCDstate.row);
  data ^= LCDstate.xor;
  data |= LCDstate.or;
  LCDdata (LCDstate.row, LCDstate.col);
  to_x -= LCDstate.col;
  to_x++;
  while (to_x) {
    LCDbyte (data);
    LCDstate.col++;
    to_x--;
  }
  LCDend ();
}
// ------------------------------------
// set LCD data masking
// ------------------------------------
void LCDmode (unsigned char x_mask, unsigned char o_mask) {
  LCDstate.xor = x_mask;
  LCDstate.or = o_mask;
}
// ------------------------------------
// set current font
// ------------------------------------
const FontType *SetFont (const FontType *fnt) {
const FontType *old;
  old = LCDstate.curr_font.ptr;
  LCDstate.curr_font.ptr=fnt;
  memcpy (&LCDstate.curr_font.header, &fnt->header, sizeof (FontHeader));
  LCDstate.curr_font.header.height=(LCDstate.curr_font.header.height+7)>>3;
  return old;
}
// ------------------------------------
// get character width
// ------------------------------------
int CharWidth (unsigned char c) {
  if (!LCDstate.curr_font.ptr) return 0;
  if (c<LCDstate.curr_font.header.min_char || c>LCDstate.curr_font.header.max_char)
    return 0;
  return LCDstate.curr_font.ptr->widths[c];
}
int StrWidth (char *s) {
int wd;
  if (!s) return 0;
  wd = 0;
  while (*s) {
    wd+=CharWidth (*s);
    s++;
  }
  return wd;
}
// ------------------------------------
// print one character at current position
// ------------------------------------
int DrawOneChar (unsigned char c) {
const unsigned char *ptr, *wdp;
unsigned char line, wd;
unsigned char r, cl;
  if (!LCDstate.curr_font.ptr) return 0;
  if (c<LCDstate.curr_font.header.min_char || c>LCDstate.curr_font.header.max_char)
    return 0;
  c-=LCDstate.curr_font.header.min_char;
  wdp=&LCDstate.curr_font.ptr->widths[LCDstate.curr_font.header.min_char];
  ptr=LCDstate.curr_font.ptr->data;
  while (c) {
    ptr+=*wdp;
    wdp++;
    c--;
  }
  r=LCDstate.row;
  cl=LCDstate.col;
  wd=(*wdp)/LCDstate.curr_font.header.height;
  line=0;
  do {
    LCDbufw (ptr, wd, LCDstate.curr_font.header.height);
    ptr++;
    line++;
    if (line<LCDstate.curr_font.header.height) LCDpos (cl, r+line);
  } while (line<LCDstate.curr_font.header.height);
  if (LCDstate.curr_font.header.height>1) LCDpos (cl+wd, r);
  return 0;
}
// ------------------------------------
// print string on current position
// ------------------------------------
void DrawChars (unsigned char x, unsigned char y, const unsigned char *s) {
unsigned char c;
  LCDpos (x, y);
  while ((c=*s)) {
    DrawOneChar (c);
    s++;
  }
}
// ------------------------------------
// print character with control code detection
// ------------------------------------
int putchar (int c) {
  if (((unsigned char)c)<' ') {
    if (c=='\f')
      LCDclear ();
    if (c=='\n')
      LCDpos (0, LCDstate.row+1);
    if (c=='\r')
      LCDpos (0, LCDstate.row);
    return 0;
  }
  // line break...
  // inter-char space can be removed (but it is not everywhere)...
  if (LCDstate.col+LCDstate.curr_font.ptr->widths[(unsigned char)c]>LCD_COLS)
    LCDpos (0, LCDstate.row+1);
  DrawOneChar (c);
  return 0;
}
// ------------------------------------
// initialization
// ------------------------------------
void LCD_init (void) {

  LCD_init_HW ();

  // basic (proportional) font setup
  //SetFont (&Font_8x6);
  SetFont (&Font_8x6_p);

  // erase display
  putchar ('\f');

}
