/*************************************************************
 * Pocket oscilloscope; (c) 2009 Hynek Sladky                *
 *                                                           *
 * main.c                                                    *
 *************************************************************/
#include <stdio.h>
#include <string.h>
#include "global.h"
#include "lcd.h"
#include "adc.h"
#include "fft.h"
// ====================================
// compile control
// ====================================
// ADC is inverted with preamplifier
#define INVERT
// backlight is off by default
#define DARK

// ====================================
// hardware init
// ------------------------------------
void HW_init (void) {

  // port init
  P1OUT = POR_P1OUT;
  P1DIR = POR_P1DIR;
  P2OUT = POR_P2OUT;
  P2DIR = POR_P2DIR;
  P3OUT = POR_P3OUT;
  P3DIR = POR_P3DIR;
  P4OUT = POR_P4OUT;
  P4DIR = POR_P4DIR;
  P5OUT = POR_P5OUT;
  P5DIR = POR_P5DIR;
  P6OUT = POR_P6OUT;
  P6DIR = POR_P6DIR;
  P6SEL = POR_P6SEL;
  // WDT off
  WDTCTL = WDTPW | WDTHOLD;
  // start 32kHz, ACLK=LFXT1/2
  //BCSCTL1 = DIVA0;
  // start 32kHz, ACLK=LFXT1/1
  BCSCTL1 = 0;
  // wait for oscillator XT1
  do
    IFG1 &= ~OFIFG; // clear oscfault flag
  while ((IFG1 & OFIFG));
  BCSCTL2 = SELM1 | SELS; // MCLK = XT2 (6.3MHz), SMCLK = XT2

  // timer init - done in LCD.C

  // TimerB: pwm...
  //TBCTL = TBSSEL1 | ID1 | MC0; // SMCLK / 4 (1MHz) up to TBCL0
  // CCR6: 1ms counter
  //TBCCR6 = 1000;
  //TBCCTL6 = CCIE;
  TBCTL = TBSSEL1 | MC0; // SMCLK / 1 (6.3MHz) up to TBCL0
  //TBCCR0 = 100; // ~ 15.75kHz
  TBCCR0 = 6300-1; // ~ 1kHz
  PWM_R_CTL = OUTMOD_6;
  PWM_G_CTL = OUTMOD_6;
  PWM_B_CTL = OUTMOD_6;
  PWM_R_CR = 0;
  PWM_G_CR = 0;
  PWM_B_CR = 0;
  P4SEL |= PWM_R_SEL | PWM_G_SEL | PWM_B_SEL;
  LED_EN_P |= LED_EN_B;
#ifdef LED_OE_P
  LED_OE_P |= LED_OE_B;
#endif
  // PWM setup for negative voltage generator
  PWM_PWR_CTL = OUTMOD_6;
  PWM_PWR_CR = 6300/2;
  P4SEL |= PWM_PWR_SEL;

  // ADC init
  // ADC on, ref int
  //ADC12CTL0 = SHT1_1 | SHT0_1 | MSC | REF2_5V | REFON | ADC12ON; // sample 8 cycles, REF 2.5V, multiple sample
  ADC12CTL0 = SHT1_1 | SHT0_1 | MSC | REFON | ADC12ON; // sample 8 cycles, REF 1.5V, multiple sample
  ADC12CTL1 = SHS_0 | SHP | ADC12SSEL_2 | CONSEQ_3; // start at address 0, start form ADC12SC, sample for SHT time, MCLK

#ifdef LH10002
  // switch LEDs off
  LED_DTA_P |= LED_DTA_B;
  for (i=5; i; i--) {
    LED_CLK_P |= LED_CLK_B;
    LED_CLK_P &= ~LED_CLK_B;
  }
  LED_LATCH_P |= LED_LATCH_B;
  LED_LATCH_P &= ~LED_LATCH_B;
#endif

}
// ====================================
// scan buttons
// ------------------------------------
static unsigned char kflags, kkey;
#define KEY_UP          0x0001
#define KEY_DOWN        0x0002
#define KEY_SEL_L       0x0004
#define KEY_SEL_R       0x0008
#ifdef SW_PORT
void ReadKeys (void) {
  kflags = 0;
  if ((KEY_PORT ^ kkey) & KEY_MASK) {
    kkey ^= KEY_PORT;
    kkey &= KEY_PORT;
    if (kkey & LEFT_MASK)
      kflags |= KEY_SEL_L;
    if (kkey & RIGHT_MASK)
      kflags |= KEY_SEL_R;
    if (kkey & DOWN_MASK)
      kflags |= KEY_DOWN;
    if (kkey & UP_MASK)
      kflags |= KEY_UP;
    kkey = KEY_PORT;
  }
}
#else
#define KeyScanRate             50
#define EncScanTime             200
#define EncSpeedL               30
#define EncSpeedH               15
// rotary encoder sequence: UP: 1-0-2-3 nebo DOWN: 1-3-2-0
// quiecent position is 2 or 3
// if kstp==4, inc/dec should be made with kidx==0 or 1
static unsigned char kidx;
static signed char kpos;
static unsigned int EncTimer;
const unsigned char kstat[4] = {1, 3, 2, 0};
void ReadKeys (void) {
unsigned char know, kstp;

  kflags = 0;
  if (EncTimer) EncTimer--;
  // button test
  if ((KEY_PORT^kkey)&KEY_MASK) {
    kkey ^= KEY_MASK;
    //if (!(kkey&KEY_MASK)) // action on release! This bug is from beginning
    if ( (kkey&KEY_MASK)) // action on press
      kflags|=KEY_SEL_R;
  }
  // rotary encoder test
  know=(INC_PORT>>INC_SHIFT)&0x03;
  // encoder used has only 2 pulses for every position
  kstp=2;
  if (know==kstat[(kidx+1)&0x03]) {
    kidx++;
    if (kpos<0) kpos=0;
    kpos++;
    if (kpos>=kstp) {
      kpos-=kstp;
      kflags|=KEY_UP;
    }
  }
  if (know==kstat[(kidx-1)&0x03]) {
    kidx--;
    if (kpos>0) kpos=0;
    kpos--;
    if (kpos<=-kstp) {
      kpos+=kstp;
      kflags|=KEY_DOWN;
    }
  }
  if (kflags&(KEY_UP|KEY_DOWN)) {
    EncTimer = EncScanTime;
  }
  return;
}
#endif
// ====================================
// DSO data structure
// ------------------------------------
typedef struct {
  int rate_sel;
//  unsigned int sample_rate;
  int offs[2];
  int div;
  int trig;
  int trig_pos;
  int trig_mode;
  int trig_edge;
  int probe;
  int vector;
  int fft_mode;
// HW based values
  int zero_offs;
  int BL_red;
  int BL_green;
  int BL_blue;
} DSO_setup_type;
// Flash setup
#define FLASH_MEM 4
const DSO_setup_type DSO_flash_setup[4] __attribute__ ((section (".infomem"))) = {
  { // default setup
//    5+5, //ADC_5ms,
    2, // ROLL 2s/div
    {0, 0/*-ADC_OFFFSET*/}, 2,
    ADC_OFFSET, 90,
    0, 0,
    1, // 1:10  //0, // 1:1
    1,
    0,
    ADC_OFFSET,
#ifdef DARK
    0, 0, 0
#else
    7, 10, 10
#endif
  },
  { // setup for scope+FFT after autoscale
    4+5,
    {1, 0}, 3,
    1027, 90,
    1, 0,
    0,
    1,
    1,
    ADC_OFFSET,
    7, 10, 10
  },
  {0},
  {0}
};

// RAM setup
DSO_setup_type DSO_setup;

int DSO_buf[128];
// ====================================
// menu items
// ------------------------------------
// menu structure definition
// ------------------------------------
typedef struct {
  char *name;
  unsigned int val;
} menu_item_type;
// ------------------------------------
// scope mode
// ------------------------------------
#define FFT_COUNT       3
const menu_item_type fft_type[FFT_COUNT] = {
  {"scope", 0},
  {"scope+fft", 1},
  {"fft", 2}
};
// ------------------------------------
// sample rate selection
// ------------------------------------
#define ROLL_COUNT      5
#define RATE_COUNT      5+11
typedef struct {
  char *name[FFT_COUNT];
  unsigned int val;
} menu_item_type_A;
const menu_item_type_A rate_sel[RATE_COUNT] = {
  {{"10s",0,0}, 40},
  {{"5s",0,0}, 20},
  {{"2s",0,0}, 8},
  {{"1s",0,0}, 4},
  {{"500ms",0,0}, 2},
  {{"200ms","35Hz","18Hz"}, ADC_200ms},
  {{"100ms","70Hz","35Hz"}, ADC_100ms},
  {{"50ms","175Hz","70Hz"}, ADC_50ms},
  {{"20ms","350Hz","175Hz"}, ADC_20ms},
  {{"10ms","700Hz","350Hz"}, ADC_10ms},
  {{"5ms","1.75kHz","700Hz"}, ADC_5ms},
  {{"2ms","3.5Hz","1.75kHz"}, ADC_2ms},
  {{"1ms","7kHz","3.5kHz"}, ADC_1ms},
  {{"500us","17.5kHz","7kHz"}, ADC_500us},
  {{"200us","35kHz","17.5kHz"}, ADC_200us},
  {{"100us","70kHz","35kHz"}, ADC_100us}
};
// ------------------------------------
// X axis unit scale (cursor)
// ------------------------------------
//FFT resolution: 30px/100ms/128/2=1.17Hz/div
#define RP  0x8000 // reduced precision flag
// every value is with 2 decimal places except RP flag which is with 1 decimal place only
// cursor_value = (cursor_pos * cur_scale[][].scale)>>8
// if (RP) -> one decimal place
// else -> two decimal places
typedef struct {
  unsigned int scale;
  char *unit;
} cur_scale_type;
cur_scale_type cur_Xscale[FFT_COUNT][RATE_COUNT] = {
  {{33.3333*256+0.5, "s"},  // 10 s/div 0.33-40.0 - ROLL
   {16.6667*256+0.5, "s"},  //  5 s/div 0.17-20.0 - ROLL
   { 6.6667*256+0.5, "s"},  //  2 s/div 0.06-8.00 - ROLL
   { 3.3333*256+0.5, "s"},  //  1 s/div 0.03-4.00 - ROLL
   { 1.6667*256+0.5, "s"},  //500ms/div 0.01-2.00 - ROLL
   {RP | (unsigned int)(66.6667*256+0.5), "ms"}, //200ms/div 6.66-800
   {RP | (unsigned int)(33.3333*256+0.5), "ms"}, //100ms/div 3.33-400
   {RP | (unsigned int)(16.6667*256+0.5), "ms"}, // 50ms/div 1.67-200
   { 66.6667*256+0.5, "ms"}, // 20ms/div 0.66-80.0
   { 33.3333*256+0.5, "ms"}, // 10ms/div 0.33-40.0
   { 16.6667*256+0.5, "ms"}, //  5ms/div 0.17-20.0
   {  6.6667*256+0.5, "ms"}, //  2ms/div 0.06-8.00
   {  3.3333*256+0.5, "ms"}, //  1ms/div 0.03-4.00
   {  1.6667*256+0.5, "ms"}, //500us/div 0.01-2.00
   {RP | (unsigned int)(66.6667*256+0.5), "us"}, //200us/div 6.67-800
   {RP | (unsigned int)(33.3333*256+0.5), "us"}},//100us/div 3.33-400
  {{0,0},
   {0,0},
   {0,0},
   {0,0},
   {0,0},
   { 58.594*256,"Hz"}, // 18Hz/div 0.58-70.3
   {RP | (unsigned int)(11.719*256+0.5),"Hz"}, // 35Hz/div 1.17-141
   {RP | (unsigned int)(23.438*256+0.5),"Hz"}, // 70Hz/div 2.34-281
   {RP | (unsigned int)(58.594*256+0.5),"Hz"}, //175Hz/div 5.86-703
   {  1.1719*256+0.5,"kHz"}, //350Hz/div 0.01-1.41
   {  2.3438*256+0.5,"kHz"}, // 700 Hz/div 0.02-2.81
   {  5.8594*256+0.5,"kHz"}, //1.75kHz/div 0.05-7.03
   { 11.7190*256+0.5,"kHz"}, //3.50kHz/div 0.12-14.1
   { 23.4380*256+0.5,"kHz"}, //7.00kHz/div 0.23-28.1
   { 58.5940*256+0.5,"kHz"}, //17.5kHz/div 0.58-70.3
   {RP | (unsigned int)(11.719*256+0.5),"kHz"}  //35.0kHz/div 1.17-141
  }
};
// ------------------------------------
// connected probe selection
// ------------------------------------
#define PROBE_COUNT     3
const menu_item_type probe_type[PROBE_COUNT] = {
  {"1:1", 0},
  {"10:1", 0},
  {"1000:1", 0}
};
// ------------------------------------
// Y axis division
// ------------------------------------
#define DIV_COUNT       7
const menu_item_type div_sel[PROBE_COUNT][DIV_COUNT] = {
  {{"10mV", 63913},
   {"20mV", 31957},
   {"50mV", 12783},
   {"100mV", 6391},
   {"200mV", 3196},
   {"500mV", 1278},
   {"1V", 639}},
  {{"100mV", 63913},
   {"200mV", 31957},
   {"500mV", 12783},
   {"1V", 6391},
   {"2V", 3196},
   {"5V", 1278},
   {"10V", 639}},
  {{"1V", 63913},
   {"2V", 31957},
   {"5V", 12783},
   {"10V", 6391},
   {"20V", 3196},
   {"50V", 1278},
   {"100V", 639}}
};
// ------------------------------------
// Y axis unit scale (Y cursor)
// ------------------------------------
cur_scale_type cur_Yscale[PROBE_COUNT][DIV_COUNT] = {
  {{36.3636*256+0.5, "mV"}, // 10mV/div 0.3636mV/px
   {72.7273*256+0.5, "mV"}, // 20mV/div 0.7272mV/px
   {RP | (unsigned int)(18.1818*256+0.5), "mV"}, // 50mV/div 1.8181mV/px
   {RP | (unsigned int)(36.3636*256+0.5), "mV"}, //100mV/div 3.6363mV/px
   {RP | (unsigned int)(72.7272*256+0.5), "mV"}, //200mV/div 7.2727mV/px
   { 1.8181*256+0.5, "V"}, //500mV/div 18.181mV/px
   { 3.6363*256+0.5, "V"}},//  1 V/div 36.363mV/px
  {{RP | (unsigned int)(36.3636*256+0.5), "mV"}, //100mV/div 3.6363mV/px
   {RP | (unsigned int)(72.7272*256+0.5), "mV"}, //200mV/div 7.2727mV/px
   { 1.8181*256+0.5, "V"}, //500mV/div 18.181mV/px
   { 3.6363*256+0.5, "V"}, //  1 V/div 36.363mV/px
   { 7.2727*256+0.5, "V"}, //  2 V/div 72.727mV/px
   {18.1818*256+0.5, "V"}, //  5 V/div 181.81mV/px
   {36.3636*256+0.5, "V"}},// 10 V/div 363.63mV/px
  {{ 3.6363*256+0.5, "V"}, //  1V/div 0.036V/px
   { 7.2727*256+0.5, "V"}, //  2V/div 0.072V/px
   {18.1818*256+0.5, "V"}, //  5V/div 0.181V/px
   {36.3636*256+0.5, "V"}, // 10V/div 0.363V/px
   {72.7272*256+0.5, "V"}, // 20V/div 0.727V/px
   {RP | (unsigned int)(18.1818*256+0.5), "V"}, // 50V/div 1.818V/px
   {RP | (unsigned int)(36.3636*256+0.5), "V"}} //100V/div 3.636V/px
};
// ------------------------------------
// trigger mode
// ------------------------------------
enum {MODE_NORMAL, MODE_AUTO, MODE_FREE, /*MODE_ROLL,*/ MODE_COUNT};
const menu_item_type trig_mode[MODE_COUNT] = {
  {"normal", ADC_NORMAL_MODE},
  {"auto", ADC_AUTO_MODE},
  {"free", ADC_FREE_MODE}
  //{"roll", ADC_ROLL_MODE}
};
// ------------------------------------
// trigger edge selection
// ------------------------------------
#define EDGE_COUNT      2
const menu_item_type trig_edge[EDGE_COUNT] = {
#ifndef INVERT
  {"rising", ADC_RISE_EDGE},
  {"falling", ADC_FALL_EDGE}
#else
  {"rising", ADC_FALL_EDGE},
  {"falling", ADC_RISE_EDGE}
#endif
};
// ------------------------------------
// cursor mode selection
// ------------------------------------
#define CUR_COUNT       3
const menu_item_type cur_type[CUR_COUNT] = {
  {"-", 0},
  {"X", 1},
  {"Y", 2}
};
// ------------------------------------
// user menu items
// ------------------------------------
// vector/pixel waveform
// ------------------------------------
#define VECTOR_COUNT    2
const menu_item_type vector_type[VECTOR_COUNT] = {
  {"pixel", 0},
  {"vector", 1}
};
// ------------------------------------
// USER menu control structure
// ------------------------------------
enum {
  USR_file, USR_load, USR_save,
  USR_probe, USR_vector, USR_zero,
  USR_BLred, USR_BLgreen, USR_BLblue,
  USR_COUNT
};
char *USR_menu[USR_COUNT] = {
  "File", "Load", "Save",
  "Probe", "Draw", "Zero",
  "BL red", "BL green", "BL blue"
};
// ------------------------------------
// menu items constant definition
// ------------------------------------
enum {
  MENU_TYPE,
  MENU_RATE, MENU_DIV, MENU_OFFS, MENU_TRIG, MENU_TRIGpos,
  MENU_TRIGmode, MENU_TRIGedge, MENU_SETfft,
  MENU_CURxy, MENU_CURc1, MENU_CURc2,
  //MENU_SETvector, MENU_SETprobe, MENU_SETfft,
  //MENU_SETnum, MENU_SETload, MENU_SETsave,
  //MENU_BLred, MENU_BLgreen, MENU_BLblue,
  MENU_USRname, MENU_USRval,
  MENU_CNT
};
// ------------------------------------
// menu LCD space definition
// ------------------------------------
const unsigned int menu_len[MENU_CNT] = {
  0, // name space
  26, 28, 18, 18, 14,  // DSO menu
  28, 26, 42, // TRIG menu
  7, 46, 52, // CURSOR menu
  //29, 28, 42, // SETUP menu
  //13, 19, 22, // SetUp menu
  //17, 18, 17, // BackLight menu
  50, 50 // USER menu
};
// ------------------------------------
// menu control structure
// ------------------------------------
#define MENU_LEVELS   4
const struct {
  unsigned int min;
  unsigned int max;
  char *name;
} _menu_[MENU_LEVELS] = {
  {MENU_RATE, MENU_TRIGpos, "DSO"},
  //{MENU_TRIGmode, MENU_TRIGedge, "TRIG"},
  {MENU_TRIGmode, MENU_SETfft, "MODE"},
  //{MENU_SETvector, MENU_SETfft, "SET"},
  {MENU_CURxy, MENU_CURc2, "CURS"},
  //{MENU_SETnum, MENU_SETsave, "SetUp"},
  //{MENU_BLred, MENU_BLblue, "LIGHT"}
  {MENU_USRname, MENU_USRval, "USER"}
};
// ====================================
// cursor variables
// ------------------------------------
int CUR_mode=0;
//int CUR_pos[2][2] = {{0, 120}, {0, 55}};
//int CUR_pos[2][2] = {{5, 50}, {5, 115}}; // Y, X
int CUR_pos[2][2] = {{0, 55}, {0, 120}}; // Y, X
// ====================================
// setup variables
// ------------------------------------
int setup_num = 0;
int USR_num = 0;
// ====================================
// drawing menu
// ------------------------------------
// support functions
// ------------------------------------
unsigned int CUR_value (unsigned int cur_pos, unsigned int cur_scale);
void PrintCursor (int num) {
cur_scale_type *scale;
int i, j;
  //printf ((CUR_mode)?"C%d=%d%s":"C%d", num+1, CUR_value (CUR_pos[CUR_mode-1][num], cur_scale[DSO_setup.fft_mode][DSO_setup.rate_sel].scale), cur_scale[DSO_setup.fft_mode][DSO_setup.rate_sel].unit);
  if (!CUR_mode) {
    //printf ("C%d", num+1);
    return;
  }
  //sprintf (TMP_buf, (CUR_mode)?"C%d=%d%s":"C%d", num+1, CUR_value (CUR_pos[CUR_mode-1][num], cur_Xscale[DSO_setup.fft_mode][DSO_setup.rate_sel].scale), cur_scale[DSO_setup.fft_mode][DSO_setup.rate_sel].unit);
  scale = (CUR_mode&1) ? &cur_Xscale[DSO_setup.fft_mode!=0][DSO_setup.rate_sel] : &cur_Yscale[DSO_setup.probe][DSO_setup.div];
  i=CUR_pos[CUR_mode&1][num];
  if (num) {
    // delta cursor
    i-=CUR_pos[CUR_mode&1][0];
  } else {
    // absolute cursor
    if (CUR_mode&1) {
      if (!DSO_setup.fft_mode)
        i-=120-DSO_setup.trig_pos;
    } else {
      //i-=DSO_setup.offs[DSO_setup.fft_mode!=0]+28
      if (DSO_setup.fft_mode)
        i-=DSO_setup.offs[1];
      else
        i-=DSO_setup.offs[0]+28;
    }
  }
  // positive or negative number processing
  TMP_buf[20]='+';
  if (i<0) {
    // negative number
    i=-i;
    TMP_buf[20]='-';
  }
  // prepare value
  sprintf ((char*)TMP_buf, "00%d", CUR_value (i, scale->scale));
  // use appropriate number of decimals
  for (i=0; TMP_buf[i]; i++);
  j=i;
  TMP_buf[i+1]=0;
  TMP_buf[i]=TMP_buf[i-1];
  i--;
  if (!(RP & scale->scale)) {
    TMP_buf[i]=TMP_buf[i-1];
    i--;
  }
  TMP_buf[i]='.';
  for (i=0; TMP_buf[i]=='0'; i++);
  if (TMP_buf[i]=='.') i--;
  if (j-i>3) {
    TMP_buf[j]=0;
    j--;
    if (TMP_buf[j]=='.') TMP_buf[j]=0;
  }
  if (num) printf ("d%s", cur_type[CUR_mode].name);
  putchar ('=');
  if (TMP_buf[20]=='-') putchar ('-');
  printf ("%s%s", &TMP_buf[i], scale->unit);
  //printf ("C=%d", CUR_pos[(CUR_mode-1)&1][num]-DSO_setup.trig_pos);
  //printf ("C=%d", CUR_pos[(CUR_mode-1)&1][num]-DSO_setup.offs[DSO_setup.fft_mode!=0]-28);
  //printf ("C%d=%s%s", num+1, &TMP_buf[i], scale->unit);
}
// ------------------------------------
// Update menu value
// ------------------------------------
void UpdateVal (int *val, int step, int min, int max) {
  *val+=step;
  if (max>min) {
    if (*val>max) *val=max;
    if (*val<min) *val=min;
  } else {
    if (*val>min) *val=max;
    if (*val<max) *val=min;
  }
}
// ------------------------------------
// main MENU function
// ------------------------------------
void DSO_menu (int menu_num, int menu_sel) {
int j, x;
  x=0;
  for (j=_menu_[menu_num].min; j<=_menu_[menu_num].max; j++) {
    // draw menu
    LCDmode ((j==menu_sel)?0xFF:0x00, 0x00);
    LCDpos (x, 7);
    LCDfill (0x00, x);
    switch (j) {
    case MENU_RATE:
      printf ("%s", rate_sel[DSO_setup.rate_sel].name[DSO_setup.fft_mode]);
      break;
    case MENU_DIV:
      printf ("%s", div_sel[DSO_setup.probe][DSO_setup.div].name);
      break;
    case MENU_OFFS:
      //printf ("%d", DSO_setup.offs);
      printf ("offs");
      break;
    case MENU_TRIG:
      //printf ("%d", DSO_setup.trig);
      printf ("Trig");
      break;
    case MENU_TRIGpos:
      //printf ("%d", DSO_setup.trig_pos);
      printf ("Dly");
      break;
    case MENU_TRIGmode:
      printf ("%s", (DSO_setup.rate_sel<ROLL_COUNT)?"roll":trig_mode[DSO_setup.trig_mode].name);
      break;
    case MENU_TRIGedge:
      if (DSO_setup.rate_sel<ROLL_COUNT) break;
      printf ("%s", trig_edge[DSO_setup.trig_edge].name);
      break;
    case MENU_SETfft:
      printf ("%s", fft_type[DSO_setup.fft_mode].name);
      break;
    case MENU_CURxy:
      printf ("%s", cur_type[CUR_mode].name);
      break;
    case MENU_CURc1:
      PrintCursor (0);
      break;
    case MENU_CURc2:
      PrintCursor (1);
      break;
    case MENU_USRname:
      printf ("%s", USR_menu[USR_num]);
      break;
    case MENU_USRval:
      switch (USR_num) {
        case USR_vector:
          printf ("%s", vector_type[DSO_setup.vector].name);
          break;
        case USR_probe:
          printf ("%s", probe_type[DSO_setup.probe].name);
          break;
        case USR_file:
          printf ("%d", setup_num);
          break;
        case USR_load:
          printf ("load");
          break;
        case USR_save:
          printf ("save");
          break;
        case USR_zero:
          printf ("%d", DSO_setup.zero_offs);
          break;
        case USR_BLred:
          printf ("%d", DSO_setup.BL_red);
          break;
        case USR_BLgreen:
          printf ("%d", DSO_setup.BL_green);
          break;
        case USR_BLblue:
          printf ("%d", DSO_setup.BL_blue);
          break;
      }
      break;
    }
    x+=menu_len[j];
    LCDfill (0x00, x-1);
  }
  LCDmode (0x00, 0x00);
  j=StrWidth (_menu_[menu_num].name);
  x=127-j;
  LCDfill (0x00, x-1);
  // nazev menu
  LCDmode ((!menu_sel)?0xFF:0x00, 0x00);
  LCDfill (0x00, x);
  printf (_menu_[menu_num].name);
  LCDfill (0x00, 127);

  LCDmode (0x00, 0x00);
}
// ====================================
// DSO output
// ------------------------------------
// grid definition
// ------------------------------------
const unsigned char grid_line[5][8] = {
  {0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF}, // vertical border
  {0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, // horizontal border
  {0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x80, 0x80}, // horizontal grid
  {0xAA, 0xAA, 0xAA, 0xAA, 0xAA, 0xAA, 0xAA, 0xAA},  // vertical grid
  //{0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80} // horizontal border
  {0x92, 0x49, 0x24, 0x92, 0x49, 0x24, 0x92, 0x49}   // vertical cursor
};
// ------------------------------------
// sample vector lines
// ------------------------------------
const unsigned char min_line[8] = {0xFF, 0xFE, 0xFC, 0xF8, 0xF0, 0xE0, 0xC0, 0x80};
const unsigned char max_line[8] = {0x01, 0x03, 0x07, 0x0F, 0x1F, 0x3F, 0x7F, 0xFF};
// ------------------------------------
// DSO trigger arrow & grid flags
// ------------------------------------
#define DSO_TRIG_ARROW_1    0x40
#define DSO_TRIG_ARROW_2    0x80
#define DSO_TRIG_ARROW_12   0xC0

#define DSO_GRID            0x40
#define DSO_CURS            0x80
// ------------------------------------
// converting sample buffer to display buffer
// drawing DSO draw buffer
// ------------------------------------
void DSO_draw (int vector) {
int i, j, min, max;
unsigned char *ptr, smpl, cmsk;
const unsigned char scp_idx[3] = {128, 67, 0};
  // data preparation
  ptr = TMP_buf;
  max = div_sel[DSO_setup.probe][DSO_setup.div].val;
  //min = ADC_convert (ADC_OFFSET, max);
  min = ADC_convert (DSO_setup.zero_offs, max);
  smpl = scp_idx[DSO_setup.fft_mode];
  // scope results conversion
  for (i=7; i<smpl; i++) {
    j = ADC_convert (DSO_buf[i], max);
#ifndef INVERT
    j -= min;
#else
    j = min-j;
#endif
    j += DSO_setup.offs[0]; // channel offset
    j += 28; // offset to center line
    if (j<0) j=0; // clip
    if (j>55) j=55; // clip
    j = 55-j; // Y coordinates mirror
    // display result
    *ptr++ = j;
    *ptr++ = j;
  }
  // FFT results conversion
  for (; i<128; i++) {
    j = ADC_convert (DSO_buf[i], max);
    j += DSO_setup.offs[1]; // channel offset
    if (j<0) j=0; // clip
    if (j>55) j=55; // clip
    j = 55-j; // Y coordinates mirror
    // display result
    *ptr++ = j;
    *ptr++ = j;
  }
  if (vector) {
    // min/max preparation for joining lines drawing
    ptr = TMP_buf;
    smpl = ptr[0];
    for (i=0; i<121; i++) {
      min = max = ptr[0];
      if (i) {
        j = (smpl+ptr[0])>>1;
        if (j > max) max = j;
        if (j < min) min = j;
      }
      if (i<120) {
        j = (ptr[0]+ptr[2])>>1;
        if (j > max) max = j;
        if (j < min) min = j;
      }
      smpl = ptr[0];
      ptr[0] = min;
      ptr[1] = max;
      ptr += 2;
    }
    // round transition
    ptr = TMP_buf;
    for (i=0; i<120; i++) {
      min = ptr[1]-ptr[0];
      max = ptr[3]-ptr[2];
      if (ptr[1] == ptr[2]) {
        // falling or constant signal
        if (min > max) {
          ptr[1]--;
          //min--;
        } else
        if (min < max) {
          ptr[2]++;
          //max--;
        } else
        if (min) {
          ptr[2]++;
          //min--;
        }
      } else
      if (ptr[0] == ptr[3]) {
        // rising or constant signal
        if (min > max) {
          ptr[0]++;
        } else
        if (min < max)
          ptr[3]--;
        else
        if (min)
          ptr[3]--;
      }
      ptr +=2;
    }
  }
  // grid preparation
  for (i=0; i<121; i+=30) {
    TMP_buf[i+i] |= DSO_GRID;
  }
  // add cursor
  if (CUR_mode) {
    i=CUR_pos[1][0];
    if (DSO_setup.fft_mode==1) {
      i++;
      i>>=1;
      i+=60;
      if (i>120) i=120;
    }
    TMP_buf[i+i] |= DSO_CURS;
    i=CUR_pos[1][1];
    if (DSO_setup.fft_mode==1) {
      i++;
      i>>=1;
      i+=60;
      if (i>120) i=120;
    }
    TMP_buf[i+i] |= DSO_CURS;
  }
  if (DSO_setup.rate_sel>=ROLL_COUNT) {
    // trigger position
    i=120-DSO_setup.trig_pos;
    if (i<0) i=0;
    if (i>120) i=120;
    i<<=1;
    TMP_buf[i+1] |= DSO_TRIG_ARROW_12;
    if (i) TMP_buf[i-1] |= DSO_TRIG_ARROW_1;
    if (i<238) TMP_buf[i+3] |= DSO_TRIG_ARROW_1;
  }
  // draw signal
  for (min=j=0; j<7; j++) {
    LCDdata (j, 2);
    ptr = TMP_buf;
    // horizontal cursor
    cmsk = 0;
    if (CUR_mode) {
      i = 55-CUR_pos[0][0];
      if ((i>>3)==j)
        cmsk |= 1<<(i&0x07);
      i = 55-CUR_pos[0][1];
      if ((i>>3)==j)
        cmsk |= 1<<(i&0x07);
    }
    for (i=0; i<121; i++) {
      // signal
      smpl = 0;
      //smpl = cmsk;
      if ((ptr[0]&0x3F)<=min+7 && (ptr[1]&0x3F)>=min) {
        //smpl |= 0xFF;
        max = (ptr[0]&0x3F) - min;
        if (max < 0) max = 0;
        smpl = min_line[max];
        max = (ptr[1]&0x3F) - min;
        if (max > 7) max = 7;
        smpl &= max_line[max];
      }
      // grid data
      if (ptr[0] & DSO_GRID) {
        smpl |= grid_line[3][j];
      } else
      if (!(i&1))
        smpl |= grid_line[2][j];
      else {
        smpl |= grid_line[1][j];
        // horizontal cursor
        smpl |= cmsk;
      }
      // vertical cursor
      if (ptr[0] & DSO_CURS)
        smpl |= grid_line[4][j];
      // horizontal cursor
      //smpl |= cmsk;
      // trigger mark
      if (!j) {
        if (ptr[1]&DSO_TRIG_ARROW_2) smpl |= 0x04; //0x02;
        if (ptr[1]&DSO_TRIG_ARROW_1) smpl |= 0x03; //0x01;
      }
      LCDbyte (smpl);
      ptr += 2;
    }
    LCDend ();
    min+=8;
  }
}
// ------------------------------------
// DSO channel offset & trigger level
// ------------------------------------
void DSO_trig_offs (void) {
int i;
int trig, offs;
const unsigned char offs_line[2] = {0x02, 0x02};
const unsigned char trig_line[2] = {0x07, 0x02};
int j;
static unsigned char mem[14];
  // basic offset position
  offs = 28 + DSO_setup.offs[0];
  if (offs < 0) offs = 0;
  if (offs > 55) offs = 55;
  offs = 55 - offs;
  // basic trigger position
  //trig = ((DSO_setup.trig - ADC_OFFSET) >> DSO_setup.div) + 28 + DSO_setup.offs[0];
#ifndef INVERT
  trig = ((DSO_setup.trig - DSO_setup.zero_offs) >> DSO_setup.div) + 28 + DSO_setup.offs[0];
#else
  trig = ((DSO_setup.zero_offs - DSO_setup.trig) >> DSO_setup.div) + 28 + DSO_setup.offs[0];
#endif
  if (trig < 0) trig = 0;
  if (trig > 55) trig = 55;
  trig = 55 - trig;
  // preparation for signal drawing
  for (i=0; i<14; i++)
    mem[i]=0;
  i = trig >> 3;
  i += i;
  j = trig & 0x07;
  if (!j) {
    mem[i] |= trig_line[0] >> 1;
    mem[i+1] |= trig_line[1] >> 1;
    if (i) {
      i-=2;
      mem[i] |= trig_line[0] << 7;
      mem[i+1] |= trig_line[1] << 7;
    }
  } else {
    mem[i] |= trig_line[0] << (j-1);
    mem[i+1] |= trig_line[1] << (j-1);
    if (i<6 && j>6) {
      i+=2;
      mem[i] |= trig_line[0] >> (9-j);
      mem[i+1] |= trig_line[1] >> (9-j);
    }
  }
  i = offs >> 3;
  i += i;
  j = offs & 0x07;
  if (!j) {
    mem[i] |= offs_line[0] >> 1;
    mem[i+1] |= offs_line[1] >> 1;
    if (i) {
      i-=2;
      mem[i] |= offs_line[0] << 7;
      mem[i+1] |= offs_line[1] << 7;
    }
  } else {
    mem[i] |= offs_line[0] << (j-1);
    mem[i+1] |= offs_line[1] << (j-1);
    if (i<6 && j>6) {
      i+=2;
      mem[i] |= offs_line[0] >> (9-j);
      mem[i+1] |= offs_line[1] >> (9-j);
    }
  }
  // draw markers
  for (i=0; i<7; i++) {
    LCDdata (i, 0);
    LCDbyte (mem[i+i]);
    LCDbyte (mem[i+i+1]);
    LCDend ();
  }
}
// ------------------------------------
// triggered signal
// ------------------------------------
void DSO_triggered (void) {
static unsigned trig_cnt;
const unsigned char TRG_symbol[4][3] = {
  {0x02, 0x02, 0x02},
  {0x01, 0x02, 0x04},
  {0x00, 0x07, 0x00},
  {0x04, 0x02, 0x01}
};
  if (trig_cnt > 3) trig_cnt = 0;
  LCDdata (0, 125); // space is from 123, ie. 5 pixels
  LCDbyte (TRG_symbol[trig_cnt][0]);
  LCDbyte (TRG_symbol[trig_cnt][1]);
  LCDbyte (TRG_symbol[trig_cnt][2]);
  LCDend ();
  trig_cnt++;
}
// ====================================
// auto-scale
// ------------------------------------
struct {
  unsigned int key_timer;
  unsigned int timebase;
  unsigned int repeat;
  unsigned int min, max;
} autoscale = {0};
// ----
int auto_scale (void) {
int j, k, h;
  if (autoscale.repeat) {
    autoscale.repeat--;
  }
  // data processing
  // min/max
  for (j=0; j<128; j++) {
    if (DSO_buf[j]>autoscale.max) autoscale.max=DSO_buf[j];
    if (DSO_buf[j]<autoscale.min) autoscale.min=DSO_buf[j];
  }
  if (autoscale.repeat) return 0;
  DSO_setup.trig = (autoscale.min+autoscale.max)>>1;
  // periodicity
  h=autoscale.max-autoscale.min;
  //h>>=4;
  h>>=2;
  if (h<8) h=8; // minimal noise level
  for (j=k=0; j<128; j++) {
    if (DSO_buf[j]+h<DSO_setup.trig && k&1)
      k++;
    if (DSO_buf[j]>DSO_setup.trig+h && !(k&1))
      k++;
    if (k>3)
      break;
  }
  // vertical resolution setup
  h = autoscale.max - autoscale.min;
  for (DSO_setup.div=0; DSO_setup.div<DIV_COUNT; DSO_setup.div++) {
    j = ADC_convert (h, div_sel[0][DSO_setup.div].val);
    if (j <= 56)
      break;
  }
  // vertical position/offset setup
  //DSO_setup.offs[0] = ADC_convert (ADC_OFFSET, div_sel[0][DSO_setup.div].val) - ADC_convert (autoscale.min, div_sel[0][DSO_setup.div].val);
#ifndef INVERT
  DSO_setup.offs[0] = ADC_convert (DSO_setup.zero_offs, div_sel[0][DSO_setup.div].val) - ADC_convert (autoscale.min, div_sel[0][DSO_setup.div].val);
  DSO_setup.offs[0] -= j>>1;
#else
  DSO_setup.offs[0] = ADC_convert (autoscale.max, div_sel[0][DSO_setup.div].val) - ADC_convert (DSO_setup.zero_offs, div_sel[0][DSO_setup.div].val);
  DSO_setup.offs[0] -= j>>1;
#endif
  return (k>3);
}
// ------------------------------------
// safe delay function
// ------------------------------------
void delay (unsigned int ms) {
  if (ms<10)
    PWM_PWR_CR = TBR;
  while (ms--) {
    PWM_PWR_CTL &= ~CCIFG;
    while (!(PWM_PWR_CTL & CCIFG));
  }
  PWM_PWR_CR = 6300/2; // because of PWM used for negative voltage generation
}
// ====================================
// Flash functions
// ------------------------------------
// Flash Write
// ------------------------------------
int FlashWrite (void *dst, void *src, unsigned len) {
unsigned char *d, *s;
  // check if dst is in INFO memory
  if (((unsigned)dst)<0x1000 || ((unsigned)dst)>0x10FF) return 1;
  if (len>256 || ((unsigned)dst)+len>0x10FF) return 2;
  __disable_interrupt ();
  d = (unsigned char*)dst;
  s = (unsigned char*)src;
  // INFO erase
  // flash clock = (257..476)kHz; SMCLK = 6.3MHz
  FCTL2 = FWKEY | FSSEL1 | (18-1); // 350kHz
  FCTL3 = FWKEY; // clear LOCK
  FCTL1 = FWKEY | ERASE;
  *d = 0; // dummy write
  // INFO write
  FCTL1 = FWKEY | WRT;
  // copy bytes
  while (len--)
    *d++=*s++;
  FCTL1 = FWKEY; // clear WRT
  FCTL3 = FWKEY | LOCK;
  return 0;
}
// ------------------------------------
// Setup load from flash
// ------------------------------------
void LoadSetup (int setup_num) {
  LCDpos (23, 3);
  printf (" Loading setup #%d ", setup_num);
  memcpy (&DSO_setup, &DSO_flash_setup[setup_num], sizeof (DSO_setup));
  delay (1000);
}
// ------------------------------------
// setup save to flash
// ------------------------------------
void SaveSetup (int setup_num) {
DSO_setup_type *ptr;
  LCDpos (20, 3);
  printf (" Saving setup #%d ", setup_num);
  // copy flash to RAM
  ptr = (DSO_setup_type*)TMP_buf;
  memcpy (ptr, DSO_flash_setup, FLASH_MEM*sizeof (DSO_setup_type));
  memcpy (&ptr[setup_num], &DSO_setup, sizeof (DSO_setup_type));
  FlashWrite ((void*)DSO_flash_setup, ptr, FLASH_MEM*sizeof (DSO_setup_type));
  delay (1000);
}
// ====================================
// main function
// ------------------------------------
int main(void) {
int menu_num, menu_sel;
int j, k;
int flags;
unsigned int auto_delay = 0;
unsigned int roll_pos = 0;
#define MENU_REDRAW   0x01
#define DSO_REDRAW    0x02
#define DSO_INIT      0x04
#define DSO_NEW_DATA  0x08
#define DSO_TRIGGERED 0x40
#define DSO_OVERFLOW  0x80

  // load setup from Flash to RAM
  memcpy (&DSO_setup, &DSO_flash_setup[0], sizeof (DSO_setup));
  HW_init ();
  LCD_init ();
  menu_sel = menu_num = 0;
  flags = DSO_INIT | MENU_REDRAW;
  // set backlight
  PWM_R_CR = 63*DSO_setup.BL_red;
  PWM_G_CR = 63*DSO_setup.BL_green;
  PWM_B_CR = 63*DSO_setup.BL_blue;
  // set div_control
  if (DSO_setup.probe)
    DIV_10_P &= ~DIV_10_B;
  else
    DIV_10_P |= DIV_10_B;

  while (1) {
    // process preceded requests
    // buffer initialization
    if (flags & DSO_INIT) {
      for (j=0; j<128; j++) {
        //DSO_buf[j] = ADC_OFFSET - DSO_setup.offs[0];
        DSO_buf[j] = DSO_setup.zero_offs - DSO_setup.offs[0];
      }
      flags |= DSO_REDRAW;
    }
    // redraw screen
    if ((flags & DSO_NEW_DATA) && DSO_setup.trig_mode == MODE_AUTO && !(flags & DSO_TRIGGERED)) {
      // 1000 ~ 20ms
      if (auto_delay < 20000)
        auto_delay += rate_sel[DSO_setup.rate_sel].val;
      if (auto_delay < 20000) {
        flags &= ~DSO_NEW_DATA;
      }
    }
    if (flags & (DSO_REDRAW | DSO_NEW_DATA)) {
      DSO_draw (DSO_setup.vector);
      if (flags & DSO_TRIGGERED)
        auto_delay = 0;
    }
    if (flags & DSO_OVERFLOW) {
      LCDpos (124, 1);
      putchar ('o');
    }
    // redraw offset/trigger
    if (flags & DSO_REDRAW) {
      //DSO_triggered ();
      DSO_trig_offs ();
    }
    // redraw menu
    if (flags & MENU_REDRAW)
      DSO_menu (menu_num, menu_sel);
    flags = 0;

    if (autoscale.key_timer) {
      // check if button SELECT is held
      while (autoscale.key_timer && (KEY_PORT & KEY_MASK)) {
        delay (1);
        autoscale.key_timer--;
      }
      if (!autoscale.key_timer) {
        // return menu to previous item
        menu_sel--;
        if (menu_sel<0) menu_sel = _menu_[menu_num].max;
        // redraw menu
        DSO_menu (menu_num, menu_sel);
        // switch previous sampling off
        if ((ADC12CTL0 & ENC))
          ADC_stop ();
        // run auto-scale
        autoscale.timebase = RATE_COUNT;
        autoscale.repeat = 20;
        autoscale.min = 4096;
        autoscale.max = 0;
        // default parameters
        DSO_setup.trig_pos = 90;
      }
      autoscale.key_timer = 0;
    }
    // set parameters for next auto-scale step
    if (autoscale.timebase>ROLL_COUNT) {
      DSO_setup.rate_sel = autoscale.timebase-1;
      if (autoscale.timebase != RATE_COUNT) DSO_setup.trig_mode = MODE_AUTO;
      else DSO_setup.trig_mode = MODE_NORMAL;
      //flags |= MENU_REDRAW;
      DSO_menu (menu_num, menu_sel);
    }

    // run sampling
    if (!(ADC12CTL0 & ENC)) {
      if (DSO_setup.rate_sel<ROLL_COUNT)
        //ADC_start (2, rate_sel[DSO_setup.rate_sel].val, ADC_OFFSET - DSO_setup.offs[0], ADC_250ms, ADC_ROLL_MODE);
        ADC_start (2, rate_sel[DSO_setup.rate_sel].val, DSO_setup.zero_offs - DSO_setup.offs[0], ADC_250ms, ADC_ROLL_MODE);
      else
        ADC_start (2, DSO_setup.trig_pos, DSO_setup.trig, rate_sel[DSO_setup.rate_sel].val, trig_mode[DSO_setup.trig_mode].val | trig_edge[DSO_setup.trig_edge].val);
    }
    // read encoder state
    ReadKeys ();
    // menu processing
    if (kflags & (KEY_SEL_L | KEY_SEL_R)) {
      // menu sequence should be: 0, .min .. .max
      if (kflags & KEY_SEL_R) {
        menu_sel++;
        if (menu_sel == 1 && menu_num)
          menu_sel = _menu_[menu_num].min;
        if (menu_sel > _menu_[menu_num].max)
          menu_sel = 0;
      } else {
        menu_sel--;
        if (menu_sel < _menu_[menu_num].min)
          menu_sel = 0;
        if (menu_sel < 0)
          menu_sel = _menu_[menu_num].max;
      }
      kflags &= ~(KEY_SEL_L | KEY_SEL_R);
      flags |= MENU_REDRAW;
      autoscale.key_timer = 700;
    }
    // menu item processing
    j=k=0;
    if (kflags & KEY_UP) {
      kflags &= ~KEY_UP;
      j++;
    } else
    if (kflags & KEY_DOWN) {
      kflags &= ~KEY_DOWN;
      j--;
    }
    if (j) {
      switch (menu_sel) {
      case MENU_TYPE:
        UpdateVal (&menu_num, j, MENU_LEVELS-1, 0);
        break;
      case MENU_RATE:
        if ((ADC12CTL0 & ENC))
          ADC_stop ();
        UpdateVal (&DSO_setup.rate_sel, j, 0, RATE_COUNT-1);
        if (DSO_setup.fft_mode && DSO_setup.rate_sel<ROLL_COUNT)
          DSO_setup.rate_sel=ROLL_COUNT;
        flags |= DSO_INIT;
        break;
      case MENU_DIV:
        UpdateVal (&DSO_setup.div, -j, 0, DIV_COUNT-1);
        flags |= DSO_REDRAW;
        break;
      case MENU_OFFS:
        UpdateVal (&DSO_setup.offs[DSO_setup.fft_mode!=0], j, -2047, 2047);
        flags |= DSO_REDRAW;
        break;
      case MENU_TRIG:
        if (DSO_setup.rate_sel<ROLL_COUNT) break;
#ifndef INVERT
        UpdateVal (&DSO_setup.trig, j << DSO_setup.div, 0, 4096);
#else
        UpdateVal (&DSO_setup.trig, (-j) << DSO_setup.div, 0, 4096);
#endif
        flags |= DSO_INIT;
        break;
      case MENU_TRIGpos:
        if (DSO_setup.rate_sel<ROLL_COUNT) break;
        UpdateVal (&DSO_setup.trig_pos, -j, 0, 2000);
        flags |= DSO_INIT;
        break;
      case MENU_TRIGmode:
        if (DSO_setup.rate_sel<ROLL_COUNT) break;
        UpdateVal (&DSO_setup.trig_mode, j, MODE_COUNT-1, 0);
        flags |= DSO_INIT;
        break;
      case MENU_TRIGedge:
        if (DSO_setup.rate_sel<ROLL_COUNT) break;
        UpdateVal (&DSO_setup.trig_edge, j, EDGE_COUNT-1, 0);
        flags |= DSO_INIT;
        break;
      case MENU_SETfft:
        if (!DSO_setup.fft_mode && DSO_setup.rate_sel<ROLL_COUNT)
          break;
        UpdateVal (&DSO_setup.fft_mode, j, FFT_COUNT-1, 0);
        //flags |= DSO_REDRAW;
        flags |= DSO_INIT;
        break;
      case MENU_CURxy:
        UpdateVal (&CUR_mode, j, CUR_COUNT-1, 0);
        flags |= DSO_REDRAW;
        break;
      case MENU_CURc2:
        k=1;
      case MENU_CURc1:
        if (!CUR_mode) break;
        UpdateVal (&CUR_pos[CUR_mode&1][k], j, 0, (CUR_mode&1)?120:55);
        if (DSO_setup.fft_mode==1)
          UpdateVal (&CUR_pos[CUR_mode&1][k], j, 0, (CUR_mode&1)?120:55);
        flags |= DSO_REDRAW;
        break;
      // USER menu
      case MENU_USRname:
        UpdateVal (&USR_num, j, USR_COUNT-1, 0);
        break;
      case MENU_USRval:
        switch (USR_num) {
          case USR_vector:
            UpdateVal (&DSO_setup.vector, j, VECTOR_COUNT-1, 0);
            flags |= DSO_REDRAW;
            break;
          case USR_probe:
            UpdateVal (&DSO_setup.probe, j, 0, PROBE_COUNT-1);
            if (DSO_setup.probe)
              DIV_10_P &= ~DIV_10_B;
            else
              DIV_10_P |= DIV_10_B;
            flags |= DSO_REDRAW;
            break;
          case USR_file:
            UpdateVal (&setup_num, j, FLASH_MEM-1, 0);
            break;
          case USR_load:
            LoadSetup (setup_num);
            flags |= DSO_REDRAW;
            break;
          case USR_save:
            SaveSetup (setup_num);
            flags |= DSO_REDRAW;
            break;
          case USR_zero:
            UpdateVal (&DSO_setup.zero_offs, -j, 0, 4095);
            // I have to modify other values to be 'relatively' same...
            UpdateVal (&DSO_setup.trig, -j, 0, 4096);
            //UpdateVal (&DSO_setup.offs[DSO_setup.fft_mode!=0], j, -2047, 2047);
            //
            flags |= DSO_REDRAW;
            break;
          case USR_BLred:
            UpdateVal (&DSO_setup.BL_red, j, 0, 99);
            PWM_R_CR=63*DSO_setup.BL_red;
            break;
          case USR_BLgreen:
            UpdateVal (&DSO_setup.BL_green, j, 0, 99);
            PWM_G_CR=63*DSO_setup.BL_green;
            break;
          case USR_BLblue:
            UpdateVal (&DSO_setup.BL_blue, j, 0, 99);
            PWM_B_CR=63*DSO_setup.BL_blue;
            break;
        }
        break;
      }
      flags |= MENU_REDRAW;
    }
    // update LCD in ROLL mode
    if (DSO_setup.rate_sel<ROLL_COUNT) {
      if (roll_pos!=ADC_pos) {
        roll_pos=ADC_pos;
        // ADC_buf is moved to DSO_buf
        ADC_get (DSO_buf);
        // DSO_buf is processed using TMP_buf
        //DSO_draw (DSO_setup.vector);
        flags |= DSO_NEW_DATA | DSO_REDRAW | DSO_TRIGGERED;
        DSO_triggered ();
      }
    } else {
      if ((j = ADC_state ())) {
        // stop sampling
        ADC_stop ();
        if (flags & MENU_REDRAW)
          j |= ADC_BUF_PART;
        // save signal
        if (!(j & ADC_BUF_PART)) {
          ADC_get (DSO_buf);
          flags |= DSO_NEW_DATA;
          if (autoscale.timebase) {
            flags |= DSO_REDRAW;
            if (auto_scale ())
              autoscale.timebase = 0;
            else
            if (!autoscale.repeat)
              autoscale.timebase--;
            if (!autoscale.timebase)
              DSO_setup.trig_mode = MODE_AUTO;
          }
          DSO_triggered ();
          if (j & ADC_TRIGGERED)
            flags |= DSO_TRIGGERED;
          // overflow flag
          if (j & ADC_ERR_OVER)
            flags |= DSO_OVERFLOW;
          // FFT
          if (DSO_setup.fft_mode)
            fft (DSO_buf, DSO_setup.fft_mode);
        }
      }
    }
  }
}
